package fr.itsonus.bousoleplussbackend.usecase;

import fr.itsonus.bousoleplussbackend.domain.auth.spi.UserCacheRepository;
import fr.itsonus.bousoleplussbackend.domain.notification.NotificationService;
import fr.itsonus.bousoleplussbackend.exception.InvalidPasswordException;
import fr.itsonus.bousoleplussbackend.exception.UnknownEmailException;
import fr.itsonus.bousoleplussbackend.security.jwt.JwtGenerator;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.net.URI;

@Service
public class UserPasswordResetUseCase {

    private final UserCacheRepository userRepository;
    private final NotificationService notificationService;

    private final String from;
    private final String changePasswordUrl;
    private final JwtGenerator jwtGenerator;
    private final PasswordEncoder passwordEncoder;

    public UserPasswordResetUseCase(final UserCacheRepository userRepository,
                                    final NotificationService notificationService,
                                    @Value("${spring.mail.from}") String from,
                                    @Value("${frontend.url}") String frontEndUrl,
                                    @Value("${frontend.change-password.path}") String changePasswordPath,
                                    JwtGenerator jwtGenerator, PasswordEncoder passwordEncoder) {
        this.userRepository = userRepository;
        this.notificationService = notificationService;
        this.from = from;
        this.changePasswordUrl = URI.create(frontEndUrl).resolve(changePasswordPath).toString();
        this.jwtGenerator = jwtGenerator;
        this.passwordEncoder = passwordEncoder;
    }

    public void notifyForPasswordReset(String email) {
        if (!userRepository.existsByEmail(email)) {
            throw new UnknownEmailException();
        }
        var token = jwtGenerator.generateResetPasswordToken(email);
        var url = String.format("%s?token=%s", changePasswordUrl, token.value());
        notificationService.notify(from, email, "Réinitialisation de votre mot de passe",
                String.format("""
                        Bonjour !
                        
                        Cliquez sur le lien suivant pour réinitialiser votre mot de passe ou copiez / coller le dans votre navigateur :
                        %s
                        """, url));
    }

    public void resetPassword(String token, String email, String newPassword, String confirmPassword) {
        var subject = jwtGenerator.getSubjectFromJWT(token);
        if (!subject.equals(email)) {
            throw new InvalidPasswordException("Invalid token provided");
        }
        if (!newPassword.equals(confirmPassword)) {
            throw new InvalidPasswordException("Saisir le même mot de passe dans les champs 'Mot de passe' et 'Confirmation du mot de passe'.");
        }
        var user = userRepository.findByEmail(email).orElseThrow(UnknownEmailException::new);
        userRepository.save(user
                .toBuilder()
                .password(passwordEncoder.encode(newPassword))
                .build()
        );
    }
}
